@@ -1,13 +1,4 @@
-
-################################################################################
-####  NOTE:  THIS IS STILL IN DEVELOPMENT:                                  ####
-####                                                                        ####
-####    - No encoder                                                        ####
-####    - Needs more tests!                                                 ####
-####                                                                        ####
-################################################################################
-
-'''
+"""
 RSON -- readable serial object notation
 
 RSON is a superset of JSON with relaxed syntax for human readability.
@@ -19,9 +10,9 @@ Simple usage example:
 Additional documentation available at:
 
 http://code.google.com/p/rson/
-'''
+"""
 
-__version__ = '0.06'
+__version__ = '0.08'
 
 __author__ = 'Patrick Maupin <pmaupin@gmail.com>'
 
@@ -54,32 +45,34 @@ import bisect
 import re
 import sys
 
+
 class RSONDecodeError(ValueError):
     pass
 
+
 class Tokenizer(list):
-    ''' The RSON tokenizer uses re.split() to rip the source string
-        apart into smaller strings which may or may not be true standalone
-        tokens.  Sufficient information is maintained to put Humpty-Dumpty
-        back together when necessary.
+    """The RSON tokenizer uses re.split() to rip the source string
+    apart into smaller strings which may or may not be true standalone
+    tokens.  Sufficient information is maintained to put Humpty-Dumpty
+    back together when necessary.
 
-        The tokens are kept in a reversed list.  This makes retrieving
-        the next token a low-cost pop() operation from the end of the list,
-        and facilitates arbitrary lookahead operations.
+    The tokens are kept in a reversed list.  This makes retrieving
+    the next token a low-cost pop() operation from the end of the list,
+    and facilitates arbitrary lookahead operations.
 
-        Each token is a tuple, containing the following elements:
-           [0] Negative character offset of token within the source string.
-               A negative offset is used so that the tokens are sorted
-               properly for bisect() for special operations.
-           [1] single-character string usually containing a character
-               which represents the token type (often the entire token).
-           [2] string containing entire token
-           [3] string (possibly null) containing whitespace after token
-           [4] Indentation value of line containing token
-                            (\n followed by 0 or more spaces)
-           [5] Line number of token
-           [6] Tokenizer object that the token belongs to (for error reporting)
-    '''
+    Each token is a tuple, containing the following elements:
+       [0] Negative character offset of token within the source string.
+           A negative offset is used so that the tokens are sorted
+           properly for bisect() for special operations.
+       [1] single-character string usually containing a character
+           which represents the token type (often the entire token).
+       [2] string containing entire token
+       [3] string (possibly null) containing whitespace after token
+       [4] Indentation value of line containing token
+                        (\n followed by 0 or more spaces)
+       [5] Line number of token
+       [6] Tokenizer object that the token belongs to (for error reporting)
+    """
 
     # Like Python, indentation is special.  I originally planned on making
     # the space character the only valid indenter, but that got messy
@@ -95,7 +88,7 @@ class Tokenizer(list):
     # RSON syntax delimiters are tokenized separately from everything else.
     delimiterset = set(' { } [ ] : = , '.split())
 
-    re_delimiterset = ''.join(delimiterset).replace(']', r'\]')
+    re_delimiterset = ''.join(delimiterset).replace(']', r'\]').replace('[', r'\[')
 
     # Create a RE pattern for the delimiters
     delimiter_pattern = '[%s]' % re_delimiterset
@@ -119,18 +112,14 @@ class Tokenizer(list):
 
     other = r'[\S](?:[^%s\n]*[^%s\s])*' % (re_delimiterset, re_delimiterset)
 
-    pattern = '(%s)' % '|'.join([
-      delimiter_pattern,
-      triple_quoted_string,
-      quoted_string,
-      other,
-      indentation,
-    ])
+    pattern = '(%s)' % '|'.join(
+        [delimiter_pattern, triple_quoted_string, quoted_string, other, indentation]
+    )
 
     splitter = re.compile(pattern).split
 
     @classmethod
-    def factory(cls, len=len, iter=iter, unicode=unicode, isinstance=isinstance):
+    def factory(cls):
         splitter = cls.splitter
         delimiterset = set(cls.delimiterset) | set('"')
 
@@ -139,56 +128,62 @@ class Tokenizer(list):
             self.client = client
 
             # Deal with 8 bit bytes for now
-            if isinstance(source, unicode):
+            if isinstance(source, str):
                 source = source.encode('utf-8')
 
             # Convert MS-DOS or Mac line endings to the one true way
-            source = source.replace('\r\n', '\n').replace('\r', '\n')
-            sourcelist = splitter(source)
+            source = source.replace(b'\r\n', b'\n').replace(b'\r', b'\n')
+            sourcelist = splitter(source.decode())
 
             # Get the indentation at the start of the file
             indentation = '\n' + sourcelist[0]
             linenum = 1
-            linestart = offset = 0
 
             # Set up to iterate over the source and add to the destination list
             sourceiter = iter(sourcelist)
-            next = sourceiter.next
-            offset -= len(next())
+            next_ = sourceiter.__next__
+
+            offset = 0 - len(next_())
 
             # Strip comment from first line
             if len(sourcelist) > 1 and sourcelist[1].startswith('#'):
                 i = 1
                 while len(sourcelist) > i and not sourcelist[i].startswith('\n'):
                     i += 1
-                    offset -= len(next())
-
+                    offset -= len(next_())
 
             # Preallocate the list
             self.append(None)
-            self *= len(sourcelist) / 2 + 1
+            self *= len(sourcelist) // 2 + 1
             index = 0
 
             # Create all the tokens
             for token in sourceiter:
-                whitespace = next()
+                whitespace = next_()
                 t0 = token[0]
                 if t0 not in delimiterset:
                     if t0 == '\n':
                         linenum += 1
                         indentation = token
                         offset -= len(token)
-                        linestart = offset
                         continue
                     else:
                         t0 = 'X'
-                self[index] = (offset, t0, token, whitespace, indentation, linenum, self)
+                self[index] = (
+                    offset,
+                    t0,
+                    token,
+                    whitespace,
+                    indentation,
+                    linenum,
+                    self,
+                )
                 index += 1
                 offset -= len(token) + len(whitespace)
 
             # Add a sentinel
             self[index] = (offset, '@', '@', '', '', linenum + 1, self)
-            self[index+1:] = []
+            self[index + 1 :] = []
 
             # Put everything we need in the actual object instantiation
             self.reverse()
@@ -196,6 +191,7 @@ class Tokenizer(list):
             self.next = self.pop
             self.push = self.append
             return self
+
         return newstring
 
     def peek(self):
@@ -206,8 +202,7 @@ class Tokenizer(list):
 
     @staticmethod
     def sourceloc(token):
-        ''' Return the source location for a given token
-        '''
+        """Return the source location for a given token"""
         lineno = token[5]
         colno = offset = -token[0] + 1
         if lineno != 1:
@@ -216,8 +211,7 @@ class Tokenizer(list):
 
     @classmethod
     def error(cls, s, token):
-        ''' error performs generic error reporting for tokens
-        '''
+        """error performs generic error reporting for tokens"""
         offset, lineno, colno = cls.sourceloc(token)
 
         if token[1] == '@':
@@ -233,6 +227,16 @@ class Tokenizer(list):
         raise err
 
 
+def make_hashable(what):
+    try:
+        hash(what)
+        return what
+    except TypeError:
+        if isinstance(what, dict):
+            return tuple(sorted(make_hashable(x) for x in what.items()))
+        return tuple(make_hashable(x) for x in what)
+
+
 class BaseObjects(object):
 
     # These hooks allow compatibility with simplejson
@@ -249,94 +253,83 @@ class BaseObjects(object):
     # Default JSON requires string keys
     disallow_nonstring_keys = True
 
-    def default_array_factory(self):
-        ''' This function returns a constructor for RSON arrays.
-        '''
-        def new_array(startlist, token):
-            return startlist
-        return new_array
+    class default_array(list):
+        def __new__(self, startlist, token):
+            return list(startlist)
 
     class default_object(dict):
-        ''' By default, RSON objects are dictionaries that
-            allow attribute access to their existing contents.
-        '''
-        def __init__(self):
-            self.__dict__ = self
+        """By default, RSON objects are dictionaries that
+        allow attribute access to their existing contents.
+        """
 
-    def default_object_factory(self, isinstance=isinstance, dict=dict, hash=hash,
-                            tuple=tuple, sorted=sorted, ValueError=ValueError):
-        ''' This function returns a constructor for RSON objects (dicts).
-        '''
+        def __getattr__(self, key):
+            return self[key]
 
-        default_object = self.default_object
+        def __setattr__(self, key, value):
+            self[key] = value
 
-        def make_hashable(what):
-            try:
-                hash(what)
-                return what
-            except TypeError:
-                if isinstance(what, dict):
-                    return tuple(sorted(make_hashable(x) for x in what.iteritems()))
-                return tuple(make_hashable(x) for x in what)
+        def append(self, itemlist):
+            mydict = self
+            value = itemlist.pop()
+            itemlist = [make_hashable(x) for x in itemlist]
+            lastkey = itemlist.pop()
 
-        def merge(source, token):
-            result = default_object()
-            for itemlist in source:
-                mydict = result
-                value = itemlist.pop()
-                itemlist = [make_hashable(x) for x in itemlist]
-                lastkey = itemlist.pop()
+            if itemlist:
+                itemlist.reverse()
+                while itemlist:
+                    key = itemlist.pop()
+                    subdict = mydict.get(key)
+                    if not isinstance(subdict, dict):
+                        subdict = mydict[key] = type(self)()
+                    mydict = subdict
+            if isinstance(value, dict):
+                oldvalue = mydict.get(lastkey)
+                if isinstance(oldvalue, dict):
+                    oldvalue.update(value)
+                    return
+            mydict[lastkey] = value
 
-                if itemlist:
-                    itemlist.reverse()
-                    while itemlist:
-                        key = itemlist.pop()
-                        subdict = mydict.get(key)
-                        if not isinstance(subdict, dict):
-                            subdict = mydict[key] = default_object()
-                        mydict = subdict
-                if isinstance(value, dict):
-                    oldvalue = mydict.get(lastkey)
-                    if isinstance(oldvalue, dict):
-                        oldvalue.update(value)
-                        continue
-                mydict[lastkey] = value
-            return result
+        def get_result(self, token):
+            return self
 
-        return merge
-
-    def object_type_factory(self, dict=dict, tuple=tuple):
-        ''' This function returns constructors for RSON objects and arrays.
-            It handles simplejson compatible hooks as well.
-        '''
+    def object_type_factory(self):
+        """This function returns constructors for RSON objects and arrays.
+        It handles simplejson compatible hooks as well.
+        """
         object_hook = self.object_hook
         object_pairs_hook = self.object_pairs_hook
 
         if object_pairs_hook is not None:
-            def build_object(source, token):
-                return object_pairs_hook([tuple(x) for x in source])
+
+            class build_object(list):
+                def get_result(self, token):
+                    return object_pairs_hook([tuple(x) for x in self])
+
             self.disallow_multiple_object_keys = True
             self.disallow_nonstring_keys = True
         elif object_hook is not None:
             mydict = dict
-            def build_object(source, token):
-                return object_hook(mydict(source))
+
+            class build_object(list):
+                def get_result(self, token):
+                    return object_hook(mydict(self))
+
             self.disallow_multiple_object_keys = True
             self.disallow_nonstring_keys = True
         else:
-            build_object = self.default_object_factory()
+            build_object = self.default_object
 
-        build_array = self.array_hook or self.default_array_factory()
+        build_array = self.array_hook or self.default_array
         return build_object, build_array
 
 
 class Dispatcher(object):
-    ''' Assumes that this is mixed-in to a class with an
-        appropriate parser_factory() method.
-    '''
+    """Assumes that this is mixed-in to a class with an
+    appropriate parser_factory() method.
+    """
 
     @classmethod
-    def dispatcher_factory(cls, hasattr=hasattr, tuple=tuple, sorted=sorted):
+    def dispatcher_factory(cls):
 
         self = cls()
         parser_factory = self.parser_factory
@@ -348,13 +341,13 @@ class Dispatcher(object):
             if not kw:
                 return default_loads(s)
 
-            key = tuple(sorted(kw.iteritems()))
+            key = tuple(sorted(kw.items()))
             func = cached(key)
             if func is None:
                 # Begin some real ugliness here -- just modify our instance to
                 # have the correct user variables for the initialization functions.
                 # Seems to speed up simplejson testcases a bit.
-                self.__dict__ = dict((x,y) for (x,y) in key if y is not None)
+                self.__dict__ = dict((x, y) for (x, y) in key if y is not None)
                 func = parsercache[key] = parser_factory()
 
             return func(s)
@@ -363,26 +356,27 @@ class Dispatcher(object):
 
 
 class QuotedToken(object):
-    ''' Subclass or replace this if you don't like quoted string handling
-    '''
+    """Subclass or replace this if you don't like quoted string handling"""
+
+    parse_encoded_chr = chr
+    parse_quoted_str = staticmethod(lambda token, s, str=str: str(s))
 
-    parse_quoted_str = staticmethod(
-          lambda token, s, unicode=unicode: unicode(s, 'utf-8'))
-    parse_encoded_chr = unichr
     parse_join_str = u''.join
     cachestrings = False
 
     quoted_splitter = re.compile(r'(\\u[0-9a-fA-F]{4}|\\.|")').split
-    quoted_mapper = { '\\\\' : u'\\',
-               r'\"' : u'"',
-               r'\/' : u'/',
-               r'\b' : u'\b',
-               r'\f' : u'\f',
-               r'\n' : u'\n',
-               r'\r' : u'\r',
-               r'\t' : u'\t'}.get
+    quoted_mapper = {
+        '\\\\': '\\',
+        r'\"': '"',
+        r'\/': '/',
+        r'\b': '\b',
+        r'\f': '\f',
+        r'\n': '\n',
+        r'\r': '\r',
+        r'\t': '\t',
+    }.get
 
-    def quoted_parse_factory(self, int=int, iter=iter, len=len):
+    def quoted_parse_factory(self):
         quoted_splitter = self.quoted_splitter
         quoted_mapper = self.quoted_mapper
         parse_quoted_str = self.parse_quoted_str
@@ -395,13 +389,15 @@ class QuotedToken(object):
 
         def badstring(token, special):
             if token[2] != '"""' or triplequoted is None:
-                token[-1].error('Invalid character in quoted string: %s' % repr(special), token)
+                token[-1].error(
+                    'Invalid character in quoted string: %s' % repr(special), token
+                )
             result = parse_quoted_str(token, triplequoted(token))
             if cachestrings:
                 result = token[-1].stringcache(result, result)
             return result
 
-        def parse(token, next):
+        def parse(token, next_):
             s = token[2]
             if len(s) < 2 or not (s[0] == s[-1] == '"'):
                 token[-1].error('No end quote on string', token)
@@ -411,16 +407,20 @@ class QuotedToken(object):
                 result = [result]
                 append = result.append
                 s = iter(s)
-                next = s.next
-                next()
+
+                next_ = s.__next__
+                next_()
+
                 for special in s:
-                    nonmatch = next()
+                    nonmatch = next_()
                     remap = quoted_mapper(special)
                     if remap is None:
                         if len(special) == 6:
                             uni = int(special[2:], 16)
-                            if 0xd800 <= uni <= 0xdbff and allow_double:
-                                uni, nonmatch = parse_double_unicode(uni, nonmatch, next, token)
+                            if 0xD800 <= uni <= 0xDBFF and allow_double:
+                                uni, nonmatch = parse_double_unicode(
+                                    uni, nonmatch, next_, token
+                                )
                             remap = parse_encoded_chr(uni)
                         else:
                             return badstring(token, special)
@@ -432,23 +432,24 @@ class QuotedToken(object):
                 result = token[-1].stringcache(result, result)
             return result
 
-
-        def parse_double_unicode(uni, nonmatch, next, token):
-            ''' Munged version of UCS-4 code pair stuff from
-                simplejson.
-            '''
+        def parse_double_unicode(uni, nonmatch, next_, token):
+            """Munged version of UCS-4 code pair stuff from
+            simplejson.
+            """
             ok = True
             try:
-                uni2 = next()
-                nonmatch2 = next()
-            except:
+                uni2 = next_()
+                nonmatch2 = next_()
+            except Exception:
                 ok = False
-            ok = ok and not nonmatch and uni2.startswith(r'\u') and len(uni2) == 6
+            ok = ok and not nonmatch and uni2.startswith(b'\\u') and len(uni2) == 6
             if ok:
                 nonmatch = uni2
-                uni = 0x10000 + (((uni - 0xd800) << 10) | (int(uni2[2:], 16) - 0xdc00))
+                uni = 0x10000 + (((uni - 0xD800) << 10) | (int(uni2[2:], 16) - 0xDC00))
                 return uni, nonmatch2
-            token[-1].error('Invalid second ch in double sequence: %s' % repr(nonmatch), token)
+            token[-1].error(
+                'Invalid second ch in double sequence: %s' % repr(nonmatch), token
+            )
 
         return parse
 
@@ -462,43 +463,41 @@ class QuotedToken(object):
             end = source.find('"""', start)
             if end < 0:
                 tokens.error('Did not find end for triple-quoted string', token)
-            if source[end-1] != '\\':
+            if source[end - 1] != '\\':
                 break
-            result.append(source[start:end-1])
+            result.append(source[start : end - 1])
             result.append('"""')
             start = end + 3
         result.append(source[start:end])
-        offset = bisect.bisect(tokens, (- end -2, ))
+        offset = bisect.bisect(tokens, (-end - 2,))
         tokens[offset:] = []
         return ''.join(result)
 
 
 class UnquotedToken(object):
-    ''' Subclass or replace this if you don't like the unquoted
-        token handling.  This is designed to be a superset of JSON:
+    """Subclass or replace this if you don't like the unquoted
+    token handling.  This is designed to be a superset of JSON:
 
-          - Integers allowed to be expressed in octal, binary, or hex
-            as well as decimal.
+      - Integers allowed to be expressed in octal, binary, or hex
+        as well as decimal.
 
-          - Integers can have embedded underscores.
+      - Integers can have embedded underscores.
 
-          - Non-match of a special token will just be wrapped as a unicode
-            string.
+      - Non-match of a special token will just be wrapped as a unicode
+        string.
 
-          - Numbers can be preceded by '+' as well s '-'
-          - Numbers can be left-zero-filled
-          - If a decimal point is present, digits are required on either side,
-            but not both sides
-    '''
+      - Numbers can be preceded by '+' as well s '-'
+      - Numbers can be left-zero-filled
+      - If a decimal point is present, digits are required on either side,
+        but not both sides
+    """
 
     use_decimal = False
-    parse_int = staticmethod(
-        lambda s: int(s.replace('_', ''), 0))
+    parse_int = staticmethod(lambda s: int(s.replace('_', ''), 0))
     parse_float = float
-    parse_unquoted_str = staticmethod(
-        lambda token, unicode=unicode: unicode(token[2], 'utf-8'))
+    parse_unquoted_str = staticmethod(lambda token: str(token[2]))
 
-    special_strings = dict(true = True, false = False, null = None)
+    special_strings = dict(true=True, false=False, null=None)
 
     unquoted_pattern = r'''
     (?:
@@ -524,8 +523,7 @@ class UnquotedToken(object):
     '''
 
     def unquoted_parse_factory(self):
-        unquoted_match = re.compile(self.unquoted_pattern,
-                        re.VERBOSE).match
+        unquoted_match = re.compile(self.unquoted_pattern, re.VERBOSE).match
 
         parse_unquoted_str = self.parse_unquoted_str
         parse_float = self.parse_float
@@ -534,9 +532,10 @@ class UnquotedToken(object):
 
         if self.use_decimal:
             from decimal import Decimal
+
             parse_float = Decimal
 
-        def parse(token, next):
+        def parse(token, next_):
             s = token[2]
             m = unquoted_match(s)
             if m is None:
@@ -551,16 +550,15 @@ class UnquotedToken(object):
 
 
 class EqualToken(object):
-    ''' Subclass or replace this if you don't like the = string handling
-    '''
+    """Subclass or replace this if you don't like the = string handling"""
 
     encode_equals_str = None
 
     @staticmethod
     def parse_equals(stringlist, indent, token):
-        ''' token probably not needed except maybe for error reporting.
-            Replace this with something that does what you want.
-        '''
+        """token probably not needed except maybe for error reporting.
+        Replace this with something that does what you want.
+        """
         # Strip any trailing whitespace to the right
         stringlist = [x.rstrip() for x in stringlist]
 
@@ -604,37 +602,45 @@ class EqualToken(object):
         if encoder is None:
             encoder = read_unquoted
 
-        def parse(firsttok, next):
+        def parse(firsttok, next_):
             tokens = firsttok[-1]
             indent, linenum = firsttok[4:6]
-            token = next()
+            token = next_()
             while token[5] == linenum:
-                token = next()
-            while  token[4] > indent:
-                token = next()
+                token = next_()
+            while token[4] > indent:
+                token = next_()
             tokens.push(token)
 
             # Get rid of \n, and indent one past =
             indent = indent[1:] + ' '
 
             bigstring = tokens.source[-firsttok[0] + 1 : -token[0]]
+            bigstring = bigstring.decode('utf-8')
             stringlist = bigstring.split('\n')
             stringlist[0] = indent + stringlist[0]
             token = list(firsttok)
-            token[1:3] = 'X', parse_equals(stringlist, indent, firsttok)
+            token[1:3] = '=', parse_equals(stringlist, indent, firsttok)
             return encoder(token, next)
 
         return parse
 
 
 class RsonParser(object):
-    ''' Parser for RSON
-    '''
+    """Parser for RSON"""
 
-    allow_trailing_commas = False
-    allow_rson_sublists = True
+    disallow_trailing_commas = True
+    disallow_rson_sublists = False
+    disallow_rson_subdicts = False
 
-    def parser_factory(self, len=len, type=type, isinstance=isinstance, list=list, basestring=basestring):
+    @staticmethod
+    def post_parse(tokens, value):
+        return value
+
+    def client_info(self, parse_locals):
+        pass
+
+    def parser_factory(self):
 
         Tokenizer = self.Tokenizer
         tokenizer = Tokenizer.factory()
@@ -644,11 +650,14 @@ class RsonParser(object):
         read_quoted = self.quoted_parse_factory()
         parse_equals = self.equal_parse_factory(read_unquoted)
         new_object, new_array = self.object_type_factory()
-        allow_trailing_commas = self.allow_trailing_commas
+        disallow_trailing_commas = self.disallow_trailing_commas
         disallow_missing_object_keys = self.disallow_missing_object_keys
-        key_handling = [disallow_missing_object_keys, self.disallow_multiple_object_keys]
+        key_handling = [
+            disallow_missing_object_keys,
+            self.disallow_multiple_object_keys,
+        ]
         disallow_nonstring_keys = self.disallow_nonstring_keys
-
+        post_parse = self.post_parse
 
         def bad_array_element(token, next):
             error('Expected array element', token)
@@ -675,10 +684,10 @@ class RsonParser(object):
                 token = next()
                 t0 = token[1]
                 if t0 == ']':
-                    if result and not allow_trailing_commas:
+                    if result and disallow_trailing_commas:
                         error('Unexpected trailing comma', token)
                     break
-                append(json_value_dispatch(t0,  bad_array_element)(token, next))
+                append(json_value_dispatch(t0, bad_array_element)(token, next))
                 delim = next()
                 t0 = delim[1]
                 if t0 == ',':
@@ -691,17 +700,17 @@ class RsonParser(object):
             return result
 
         def read_json_dict(firsttok, next):
-            result = []
+            result = new_object()
             append = result.append
             while 1:
                 token = next()
                 t0 = token[1]
-                if t0  == '}':
-                    if result and not allow_trailing_commas:
+                if t0 == '}':
+                    if result and disallow_trailing_commas:
                         error('Unexpected trailing comma', token)
                     break
                 key = json_value_dispatch(t0, bad_dict_key)(token, next)
-                if disallow_nonstring_keys and not isinstance(key, basestring):
+                if disallow_nonstring_keys and not isinstance(key, str):
                     error('Non-string key %s not supported' % repr(key), token)
                 token = next()
                 t0 = token[1]
@@ -720,7 +729,7 @@ class RsonParser(object):
                         error('Unterminated dict (no matching "}")', firsttok)
                     error('Expected "," or "}"', delim)
                 break
-            return new_object(result, firsttok)
+            return result.get_result(firsttok)
 
         def read_rson_unquoted(firsttok, next):
             toklist = []
@@ -741,17 +750,27 @@ class RsonParser(object):
             result[2] = ''.join(s)
             return read_unquoted(result, next)
 
-        json_value_dispatch = {'X':read_unquoted, '[':read_json_array,
-                               '{': read_json_dict, '"':read_quoted}.get
+        json_value_dispatch = {
+            'X': read_unquoted,
+            '[': read_json_array,
+            '{': read_json_dict,
+            '"': read_quoted,
+        }.get
 
+        rson_value_dispatch = {
+            'X': read_rson_unquoted,
+            '[': read_json_array,
+            '{': read_json_dict,
+            '"': read_quoted,
+            '=': parse_equals,
+        }
 
-        rson_value_dispatch = {'X':read_rson_unquoted, '[':read_json_array,
-                                  '{': read_json_dict, '"':read_quoted,
-                                   '=': parse_equals}
-
-        if not self.allow_rson_sublists:
+        if self.disallow_rson_sublists:
             rson_value_dispatch['['] = read_rson_unquoted
 
+        if self.disallow_rson_subdicts:
+            rson_value_dispatch['{'] = read_rson_unquoted
+
         rson_key_dispatch = rson_value_dispatch.copy()
         if disallow_missing_object_keys:
             del rson_key_dispatch['=']
@@ -759,7 +778,7 @@ class RsonParser(object):
         rson_value_dispatch = rson_value_dispatch.get
         rson_key_dispatch = rson_key_dispatch.get
 
-        empty_object = new_object([], None)
+        empty_object = new_object().get_result(None)
         empty_array = new_array([], None)
         empty_array_type = type(empty_array)
         empties = empty_object, empty_array
@@ -776,9 +795,13 @@ class RsonParser(object):
                         stack.append(token)
                         lastitem = result[-1]
                         if lastitem == empty_array:
-                            result[-1], token = parse_recurse_array(stack, next, token, lastitem)
+                            result[-1], token = parse_recurse_array(
+                                stack, next, token, lastitem
+                            )
                         elif lastitem == empty_object:
-                            result[-1], token = parse_recurse_dict(stack, next, token, [])
+                            result[-1], token = parse_recurse_dict(
+                                stack, next, token, lastitem
+                            )
                         else:
                             result = None
                     if result:
@@ -790,14 +813,20 @@ class RsonParser(object):
                     bad_indent(token, next)
                 if newlinenum <= linenum:
                     if token[1] in '=:':
-                        error('Cannot mix list elements with dict (key/value) elements', token)
-                    error('Array elements must either be on separate lines or enclosed in []', token)
+                        error(
+                            'Cannot mix list elements with dict (key/value) elements',
+                            token,
+                        )
+                    error(
+                        'Array elements must either be on separate lines or enclosed in []',
+                        token,
+                    )
                 linenum = newlinenum
                 value = rson_value_dispatch(token[1], bad_top_value)(token, next)
                 result.append(value)
                 token = next()
 
-        def parse_one_dict_entry(stack, next, token, entry):
+        def parse_one_dict_entry(stack, next, token, entry, mydict):
             arrayindent, linenum = stack[-1][4:6]
             while token[1] == ':':
                 tok1 = next()
@@ -826,22 +855,29 @@ class RsonParser(object):
                     if type(entry[-1]) is type(value):
                         entry[-1] = value
                     else:
-                        error('Cannot load %s into %s' % (type(value), type(entry[-1])), stack[-1])
+                        error(
+                            'Cannot load %s into %s' % (type(value), type(entry[-1])),
+                            stack[-1],
+                        )
                 elif len(value) == 1 and type(value) is empty_array_type:
                     entry.extend(value)
                 else:
                     entry.append(value)
                 stack.pop()
             length = len(entry)
-            if length != 2  and key_handling[length > 2]:
+            if length != 2 and key_handling[length > 2]:
                 if length < 2:
                     error('Expected ":" or "=", or indented line', token)
-                error("rson client's object handlers do not support chained objects", token)
+                error(
+                    "rson client's object handlers do not support chained objects",
+                    token,
+                )
             if disallow_nonstring_keys:
                 for key in entry[:-1]:
-                    if not isinstance(key, basestring):
+                    if not isinstance(key, str):
                         error('Non-string key %s not supported' % repr(key), token)
-            return entry, token
+            mydict.append(entry)
+            return token
 
         def parse_recurse_dict(stack, next, token, result):
             arrayindent = stack[-1][4]
@@ -849,19 +885,18 @@ class RsonParser(object):
                 thisindent = token[4]
                 if thisindent != arrayindent:
                     if thisindent < arrayindent:
-                        return new_object(result, token), token
+                        return result.get_result(token), token
                     bad_unindent(token, next)
                 key = rson_key_dispatch(token[1], bad_top_value)(token, next)
                 stack[-1] = token
-                entry, token = parse_one_dict_entry(stack, next, next(), [key])
-                result.append(entry)
+                token = parse_one_dict_entry(stack, next, next(), [key], result)
 
-        def parse_recurse(stack, next):
-            ''' parse_recurse ALWAYS returns a list or a dict.
-                (or the user variants thereof)
-                It is up to the caller to determine that it was an array
-                of length 1 and strip the contents out of the array.
-            '''
+        def parse_recurse(stack, next, tokens=None):
+            """parse_recurse ALWAYS returns a list or a dict.
+            (or the user variants thereof)
+            It is up to the caller to determine that it was an array
+            of length 1 and strip the contents out of the array.
+            """
             firsttok = stack[-1]
             value = rson_value_dispatch(firsttok[1], bad_top_value)(firsttok, next)
 
@@ -869,35 +904,51 @@ class RsonParser(object):
             # is at the same indentation, or the current value is an empty list
 
             token = next()
-            if (token[5] != firsttok[5] and
-                    (token[4] <= firsttok[4] or
-                     value in empties) and disallow_missing_object_keys):
-                return parse_recurse_array(stack, next, token, new_array([value], firsttok))
+            if (
+                token[5] != firsttok[5]
+                and (token[4] <= firsttok[4] or value in empties)
+                and disallow_missing_object_keys
+            ):
+                result = new_array([value], firsttok)
+                if tokens is not None:
+                    tokens.top_object = result
+                return parse_recurse_array(stack, next, token, result)
 
             # Otherwise, return a dict
-            entry, token = parse_one_dict_entry(stack, next, token, [value])
-            return parse_recurse_dict(stack, next, token, [entry])
-
+            result = new_object()
+            if tokens is not None:
+                tokens.top_object = result
+            token = parse_one_dict_entry(stack, next, token, [value], result)
+            return parse_recurse_dict(stack, next, token, result)
 
         def parse(source):
             tokens = tokenizer(source, None)
             tokens.stringcache = {}.setdefault
+            tokens.client_info = client_info
             next = tokens.next
-            value, token = parse_recurse([next()], next)
+            value, token = parse_recurse([next()], next, tokens)
             if token[1] != '@':
                 error('Unexpected additional data', token)
 
             # If it's a single item and we don't have a specialized
             # object builder, just strip the outer list.
-            if (len(value) == 1 and isinstance(value, list)
-                   and disallow_missing_object_keys):
+            if (
+                len(value) == 1
+                and isinstance(value, list)
+                and disallow_missing_object_keys
+            ):
                 value = value[0]
-            return value
+            return post_parse(tokens, value)
+
+        client_info = self.client_info(locals())
 
         return parse
 
 
-class RsonSystem(RsonParser, UnquotedToken, QuotedToken, EqualToken, Dispatcher, BaseObjects):
+class RsonSystem(
+    RsonParser, UnquotedToken, QuotedToken, EqualToken, Dispatcher, BaseObjects
+):
     Tokenizer = Tokenizer
 
+
 loads = RsonSystem.dispatcher_factory()
