@@ -13,10 +13,14 @@ import shutil
 import subprocess
 import tempfile
 
-import fitz
 from packaging import version
 import pytest
 
+try:
+    import fitz
+except ImportError:
+    import fitz_old as fitz
+
 
 ROOT_DIR = os.path.realpath(os.path.dirname(__file__))
 INPUT_DIR = os.path.join(ROOT_DIR, 'input')
@@ -179,7 +183,7 @@ class Item(pytest.Item):
 
     def _fail(self, msg, output=None):
         pytest.fail(
-            f'{msg}:\n\n{output.decode("utf-8")}' if output else msg,
+            f'{msg}: \n\n{output.decode("utf-8")}' if output else msg,
             pytrace=False,
         )
 
@@ -428,7 +432,7 @@ def pytest_collect_file(file_path, parent):
 
     if file_path.suffix == '.rst' and parent_dir == 'input':
         return RstFile.from_parent(parent=parent, path=file_path)
-    elif file_path.stem == 'conf.py' and parent_dir.startswith('sphinx'):
+    elif file_path.name == 'conf.py' and parent_dir.startswith('sphinx'):
         return SphinxFile.from_parent(parent=parent, path=file_path)
 
 
